package com.loafle.overflow.container.general.client;

import java.util.HashMap;
import java.util.Map;

import com.loafle.overflow.container.general.service.Service;
import com.loafle.overflow.container.client.ContainerClient;
import com.loafle.overflow.core.annotation.RPCService;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.ApplicationContext;
import org.springframework.stereotype.Component;

import static com.loafle.overflow.core.interfaces.Service.execServices;
import static com.loafle.overflow.core.interfaces.Service.ServiceMethodType;

/**
 * GeneralContainerClient
 */
@Component
public class GeneralContainerClient extends ContainerClient {
  @Autowired
  private ApplicationContext applicationContext;

  private Map<Class<?>, Object> services;

  protected void init() throws Exception {
    super.init();

    Map<String, Object> services = this.applicationContext.getBeansWithAnnotation(RPCService.class);
    if (null == services) {
      return;
    }

    this.services = new HashMap<>();
    services.forEach((name, bean) -> {
      this.services.put(bean.getClass(), bean);
    });

    execServices(this.services, ServiceMethodType.INIT, Service.OrderedServices, false);

  }
  protected void onStart() throws Exception {
    super.onStart();

    execServices(this.services, ServiceMethodType.START, Service.OrderedServices, false);
  }
  protected void onStop() throws Exception {
    execServices(this.services, ServiceMethodType.STOP, Service.OrderedServices, true);

    super.onStop();
  }
  protected void destroy() throws Exception {
    execServices(this.services, ServiceMethodType.DESTROY, Service.OrderedServices, true);

    super.destroy();
  }
}