package com.loafle.overflow.container.general.client;

import java.net.URI;
import java.util.Map;

import com.loafle.commons.server.websocket.client.Client;
import com.loafle.overflow.config.container.ContainerProtocol;
import com.loafle.overflow.config.probe.ContainerType;
import com.loafle.overflow.container.Container;
import com.loafle.overflow.container.general.crawler.Crawlers;
import com.loafle.overflow.crawler.Crawler;

import org.codehaus.jackson.map.DeserializationConfig;
import org.codehaus.jackson.map.ObjectMapper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.ComponentScan;
import org.springframework.context.annotation.Configuration;

import io.netty.handler.codec.http.DefaultHttpHeaders;
import io.netty.handler.codec.http.HttpHeaders;
import io.netty.handler.codec.http.websocketx.WebSocketClientHandshaker;
import io.netty.handler.codec.http.websocketx.WebSocketClientHandshakerFactory;
import io.netty.handler.codec.http.websocketx.WebSocketVersion;

/**
 * GeneralContainerConfiguration
 */
@Configuration
@ComponentScan(basePackages = { "com.loafle.overflow" })
public class GeneralContainerConfiguration {
  @Autowired
  @Qualifier(Client.SERVER_URI)
  protected URI serverURI;

  @Bean(Client.HANDSHAKER)
  public WebSocketClientHandshaker handshaker() {
    HttpHeaders customHeaders = new DefaultHttpHeaders();
    customHeaders.add(ContainerProtocol.HTTPRequestHeaderKey_Container_Method, ContainerProtocol.HTTPRequestHeaderValue_Container_Method_Connect);
    customHeaders.add(ContainerProtocol.HTTPRequestHeaderKey_Container_Type, ContainerType.GENERAL.toString());

    return WebSocketClientHandshakerFactory.newHandshaker(serverURI, WebSocketVersion.V13, null, false, customHeaders);
  }

  @Bean(Container.CRAWLERS)
  public Map<String, Crawler> crawlers() {
    return Crawlers.getCrawlers();
  }

  @Bean
  public ObjectMapper objectMapper() {
    ObjectMapper objectMapper = new ObjectMapper();
    objectMapper.configure(DeserializationConfig.Feature.FAIL_ON_UNKNOWN_PROPERTIES, false);
    return objectMapper;
  }
}